﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;

namespace TestProject;

/// <summary>
/// 화살표 버튼
/// </summary>
public class ArrowButton : Button
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Dependency Property
    ////////////////////////////////////////////////////////////////////////////////////////// Static
    //////////////////////////////////////////////////////////////////////////////// Public

    #region 배경 원 스트로크 두께 속성 - BackgroundEllipseStrokeThicknessProperty

    /// <summary>
    /// 배경 원 두께
    /// </summary>
    public static readonly DependencyProperty BackgroundEllipseStrokeThicknessProperty = DependencyProperty.Register
    (
        "BackgroundEllipseStrokeThickness",
        typeof(Thickness),
        typeof(ArrowButton),
        new PropertyMetadata(new Thickness(0))
    );

    #endregion
    #region 배경 원 스트로크 속성 - BackgroundEllipseStrokeProperty

    /// <summary>
    /// 배경 원 패스 데이터 속성
    /// </summary>
    public static readonly DependencyProperty BackgroundEllipseStrokeProperty = DependencyProperty.Register
    (
        "BackgroundEllipseStroke",
        typeof(Brush),
        typeof(ArrowButton),
        new PropertyMetadata(Brushes.Transparent)
    );

    #endregion
    #region 배경 원 채우기 속성 - BackgroundEllipseFillProperty

    /// <summary>
    /// 배경 원 채우기 속성
    /// </summary>
    public static readonly DependencyProperty BackgroundEllipseFillProperty = DependencyProperty.Register
    (
        "BackgroundEllipseFill",
        typeof(Brush),
        typeof(ArrowButton),
        new PropertyMetadata(Brushes.Transparent)
    );

    #endregion
    #region 화살표 크기 속성 - ArrowSizeProperty

    /// <summary>
    /// 화살표 크기 속성
    /// </summary>
    public static readonly DependencyProperty ArrowSizeProperty = DependencyProperty.Register
    (
        "ArrowSize",
        typeof(double),
        typeof(ArrowButton),
        new PropertyMetadata(20.0)
    );

    #endregion
    #region 화살표 방향 속성 - ArrowDirectionProperty

    /// <summary>
    /// 화살표 방향 속성
    /// </summary>
    public static readonly DependencyProperty ArrowDirectionProperty = DependencyProperty.Register
    (
        "ArrowDirection",
        typeof(ArrowDirection),
        typeof(ArrowButton),
        new PropertyMetadata(ArrowDirection.Left, DirectionPropertyChangedCallback)
    );

    #endregion
    #region 화살표 스트로크 두께 속성 - ArrowStrokeThicknessProperty

    /// <summary>
    /// 화살표 두께
    /// </summary>
    public static readonly DependencyProperty ArrowStrokeThicknessProperty = DependencyProperty.Register
    (
        "ArrowStrokeThickness",
        typeof(Thickness),
        typeof(ArrowButton),
        new PropertyMetadata(new Thickness(1))
    );

    #endregion
    #region 화살표 스트로크 속성 - ArrowStrokeProperty

    /// <summary>
    /// 화살표 패스 데이터 속성
    /// </summary>
    public static readonly DependencyProperty ArrowStrokeProperty = DependencyProperty.Register
    (
        "ArrowStroke",
        typeof(Brush),
        typeof(ArrowButton),
        new PropertyMetadata(Brushes.Black)
    );

    #endregion
    #region 왼쪽 화살표 패스 데이터 속성 - LeftArrowPathDataProperty

    /// <summary>
    /// 왼쪽 화살표 패스 데이터 속성
    /// </summary>
    public static readonly DependencyProperty LeftArrowPathDataProperty = DependencyProperty.Register
    (
        "LeftArrowPathData",
        typeof(string),
        typeof(ArrowButton),
        new PropertyMetadata("M 15 10 L 5 10 M 5 10 L 10 5 M 5 10 L 10 15", ArrowPathDataPropertyChangedCallback)
    );

    #endregion
    #region 오른쪽 화살표 패스 데이터 속성 - RightArrowPathDataProperty

    /// <summary>
    /// 오른쪽 화살표 패스 데이터 속성
    /// </summary>
    public static readonly DependencyProperty RightArrowPathDataProperty = DependencyProperty.Register
    (
        "RightArrowPathData",
        typeof(string),
        typeof(ArrowButton),
        new PropertyMetadata("M 5 10 L 15 10 M 15 10 L 10 5 M 15 10 L 10 15", ArrowPathDataPropertyChangedCallback)
    );

    #endregion
    #region 화살표 패스 데이터 속성 - ArrowPathDataProperty

    /// <summary>
    /// 화살표 패스 데이터 속성
    /// </summary>
    public static readonly DependencyProperty ArrowPathDataProperty = DependencyProperty.Register
    (
        "ArrowPathData",
        typeof(string),
        typeof(ArrowButton),
        new PropertyMetadata("M 15 10 L 5 10 M 5 10 L 10 5 M 5 10 L 10 15")
    );

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Property
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 배경 원 채우기 - BackgroundEllipseFill

    /// <summary>
    /// 배경 원 채우기
    /// </summary>
    public Brush BackgroundEllipseFill
    {
        get => (Brush)GetValue(BackgroundEllipseFillProperty);
        set => SetValue(BackgroundEllipseFillProperty, value);
    }

    #endregion
    #region 배경 원 스트로크 두께 - BackgroundEllipseStrokeThickness

    /// <summary>
    /// 배경 원 스트로크 두께
    /// </summary>
    public Thickness BackgroundEllipseStrokeThickness
    {
        get => (Thickness)GetValue(BackgroundEllipseStrokeThicknessProperty);
        set => SetValue(BackgroundEllipseStrokeThicknessProperty, value);
    }

    #endregion
    #region 배경 원 스트로크 - BackgroundEllipseStroke

    /// <summary>
    /// 배경 원 스트로크
    /// </summary>
    public Brush BackgroundEllipseStroke
    {
        get => (Brush)GetValue(BackgroundEllipseStrokeProperty);
        set => SetValue(BackgroundEllipseStrokeProperty, value);
    }

    #endregion
    #region 화살표 크기 - ArrowSize

    /// <summary>
    /// 화살표 크기
    /// </summary>
    public double ArrowSize
    {
        get => (double)GetValue(ArrowSizeProperty);
        set => SetValue(ArrowSizeProperty, value);
    }

    #endregion
    #region 화살표 방향 - ArrowDirection

    /// <summary>
    /// 화살표 방향
    /// </summary>
    public ArrowDirection ArrowDirection
    {
        get => (ArrowDirection)GetValue(ArrowDirectionProperty);
        set => SetValue(ArrowDirectionProperty, value);
    }

    #endregion
    #region 화살표 스트로크 두께 - ArrowStrokeThickness

    /// <summary>
    /// 화살표 스트로크 두께
    /// </summary>
    public Thickness ArrowStrokeThickness
    {
        get => (Thickness)GetValue(ArrowStrokeThicknessProperty);
        set => SetValue(ArrowStrokeThicknessProperty, value);
    }

    #endregion
    #region 화살표 스트로크 - ArrowStroke

    /// <summary>
    /// 화살표 스트로크
    /// </summary>
    public Brush ArrowStroke
    {
        get => (Brush)GetValue(ArrowStrokeProperty);
        set => SetValue(ArrowStrokeProperty, value);
    }

    #endregion
    #region 왼쪽 화살표 패스 데이터 - LeftArrowPathData

    /// <summary>
    /// 왼쪽 화살표 패스 데이터
    /// </summary>
    public string LeftArrowPathData
    {
        get => (string)GetValue(LeftArrowPathDataProperty);
        set => SetValue(LeftArrowPathDataProperty, value);
    }

    #endregion
    #region 오른쪽 화살표 패스 데이터 - RightArrowPathData

    /// <summary>
    /// 오른쪽 화살표 패스 데이터
    /// </summary>
    public string RightArrowPathData
    {
        get => (string)GetValue(RightArrowPathDataProperty);
        set => SetValue(RightArrowPathDataProperty, value);
    }

    #endregion
    #region 화살표 패스 데이터 - ArrowPathData

    /// <summary>
    /// 화살표 패스 데이터
    /// </summary>
    public string ArrowPathData
    {
        get => (string)GetValue(ArrowPathDataProperty);
        private set => SetValue(ArrowPathDataProperty, value);
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
    ////////////////////////////////////////////////////////////////////////////////////////// Static

    #region 생성자 - ArrowButton()

    /// <summary>
    /// 생성자
    /// </summary>
    static ArrowButton()
    {
        DefaultStyleKeyProperty.OverrideMetadata(typeof(ArrowButton), new FrameworkPropertyMetadata(typeof(ArrowButton)));
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Static
    //////////////////////////////////////////////////////////////////////////////// Private

    #region 방향 속성 변경시 콜백 처리하기 - DirectionPropertyChangedCallback(d, e)

    /// <summary>
    /// 방향 속성 변경시 콜백 처리하기
    /// </summary>
    /// <param name="d">이벤트 발생자</param>
    /// <param name="e">이벤트 인자</param>
    private static void DirectionPropertyChangedCallback(DependencyObject d, DependencyPropertyChangedEventArgs e)
    {
        if(d is ArrowButton button)
        {
            button.UpdateArrowPathData();
        }
    }

    #endregion
    #region 화살표 패스 데이터 속성 변경시 콜백 처리하기 - ArrowPathDataPropertyChangedCallback(d, e)

    /// <summary>
    /// 화살표 패스 데이터 속성 변경시 콜백 처리하기
    /// </summary>
    /// <param name="d">이벤트 발생자</param>
    /// <param name="e">이벤트 인자</param>
    private static void ArrowPathDataPropertyChangedCallback(DependencyObject d, DependencyPropertyChangedEventArgs e)
    {
        if(d is ArrowButton button)
        {
            button.UpdateArrowPathData();
        }
    }

    #endregion

    ////////////////////////////////////////////////////////////////////////////////////////// Instance
    //////////////////////////////////////////////////////////////////////////////// Private

    #region 화살표 패스 데이터 업데이트하기 - UpdateArrowPathData()

    /// <summary>
    /// 화살표 패스 데이터 업데이트하기
    /// </summary>
    private void UpdateArrowPathData()
    {
        ArrowPathData = ArrowDirection == ArrowDirection.Left ? LeftArrowPathData : RightArrowPathData;
    }

    #endregion
}